<?php

    require_once('db_info.php');

    class DBHelper {
        private $connection = false;
        
        // init connection and setup tables if not done already
        function __construct() {
            $this->connection = new mysqli(DB_HOST, DB_USER, DB_PASS, DB_NAME);
            if (!$this->connection) return false;
            if (!$this->is_setup()) {
                if(!$this->setup()) {
                    return $this->connection->error;
                }
            }
        }
    
        // check if tables exists
        function is_setup() {
            if (!$this->connection) return false;
            $tables = array('photos', 'users', 'comments');
            foreach ($tables as $table) {
                $query = $this->query('SHOW TABLES LIKE \''.$table.'\'');
                if (!$query || $query->num_rows < 1) return false;
            }
            return true;
        }

        // init the tables
        function setup() {
            if (!$this->connection) return false;
            $queries = array(
                'DROP TABLE IF EXISTS comments;',
                'DROP TABLE IF EXISTS photos;',
                'DROP TABLE IF EXISTS users;',
                'CREATE TABLE users (
                    id INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
                    username VARCHAR(255) UNIQUE NOT NULL,
                    password VARCHAR(255) NOT NULL,
                    token VARCHAR(255) DEFAULT NULL
                );',
                'INSERT INTO users (username, password) VALUES (\'dummy\', \'geaqrrrrr\')',
                'CREATE TABLE photos (
                    id INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
                    path VARCHAR(255) NOT NULL UNIQUE,
                    author INT NOT NULL,
                    FOREIGN KEY (author) REFERENCES users (id) ON DELETE CASCADE,
                    uploaded INT NOT NULL DEFAULT 0,
                    title VARCHAR(255)
                );',
                'INSERT INTO photos (path, author, title, uploaded) VALUES (\'img01.jpg\', 1, \'some dumb title\', '.time().')',
                'CREATE TABLE comments (
                    id INT PRIMARY KEY NOT NULL AUTO_INCREMENT,
                    user INT NOT NULL,
                    FOREIGN KEY (user) REFERENCES users (id) ON DELETE CASCADE,
                    photo INT NOT NULL,
                    FOREIGN KEY (photo) REFERENCES photos (id) ON DELETE CASCADE,
                    CONTENT VARCHAR(4096),
                    CREATED INT NOT NULL
                );');
            foreach ($queries as $query) {
                $query = $this->query($query);
                if (!$query) return false;
            }
            return true;
        }
        
        // getter for private db connection
        function connection() {
            return $this->connection;
        }
        
        // query shorthand
        function query($query) {
            return $this->connection->query($query);
        }
        
        // auto-close db on destruct
        function __destruct() {
            $this->connection->close();
        }
        
    }
    
    class AuthHelper {
        private $db;
        private $errors = array();
        private $logged_in = false;
        private $user = NULL; 
        
        function __construct($db) {
            $this->db = $db;
        }
        
        // getter for errors array
        function errors() {
            return $this->errors;
        }
        
        // validates post input and registers user
        // sets errors array and returns false in case of an error
        function register($post) {
            if (isset($post['username']) &&
                isset($post['password']) &&
                isset($post['password2'])) {
                
                $valid = true;
                if (empty($post['username'])) {
                    $valid = false;
                    $this->errors['username'] = 'This field is required.';
                } else {
                    $cleaned_data['username'] = $this->db->connection()->real_escape_string($post['username']);
                }
                if (empty($post['password'])) {
                    $valid = false;
                    $this->errors['password'] = 'This field is required.';
                } else if (strlen($post['password']) < 8) {
                    $valid = false;
                    $this->errors['password'] = 'Password should be 8 characters or more.';
                } else {
                    $cleaned_data['password'] = $this->db->connection()->real_escape_string($post['password']);
                }
                if (empty($post['password2'])) {
                        $valid = false;
                        $this->errors['password2'] = 'This field is required.';
                } else {
                    $cleaned_data['password2'] = $this->db->connection()->real_escape_string($post['password2']);
                }
                if (isset($cleaned_data['password']) && isset($cleaned_data['password2']) &&
                    ($cleaned_data['password'] != $cleaned_data['password2'])) {
                    $valid = false;
                    if (isset($this->errors['password2'])) {
                        $this->errors['password2'] .= '<br>Passwords don\'t match.';
                    } else {
                        $this->errors['password2'] = 'Passwords don\'t match.';
                    }
                }
                
                if ($valid) {
                    $query = 'SELECT * FROM users  WHERE username = \''.$cleaned_data['username'].'\'';
                    $query = $this->db->query($query);
                    if (!$query) {
                        $this->errors['database'] = 'Error on database connection.';
                        return false;
                    }
                    if ($query->num_rows > 0) {
                        $this->errors['username'] = 'User with this username exists.';
                        return false;
                    }
                    $hash = password_hash($cleaned_data['password'], PASSWORD_DEFAULT);
                    $query = 'INSERT INTO users (username, password) VALUES (\''.
                        $cleaned_data['username'].'\', \''.$hash.'\');';
                    $query = $this->db->query($query);
                    if (!$query) {
                        $this->errors['database'] = 'Error while writing in database : '.$this->db->connection()->error;
                    } else {
                        return true;
                    }
                }
            }
            return false;
        }
        
        // validates data and logs a user in
        // updates the local user
        function login($post) {
            if (isset($post['username']) &&
                isset($post['password'])) {
                
                $cleaned_data = array();
                $cleaned_data['username'] = $this->db->connection()->real_escape_string($post['username']);
                $cleaned_data['password'] = $this->db->connection()->real_escape_string($post['password']);
                
                $query = 'SELECT * FROM users WHERE username = \''.$cleaned_data['username'].'\';';
                $query = $this->db->query($query);
                if (!$query) {
                    $this->errors['database'] = 'Error on database connection.';
                    return false;
                }
                if ($query->num_rows < 1) {
                    $this->errors['username'] = 'No user with that username could be found.';
                    return false;
                }
                $this->user = $query->fetch_object();
                $hash = $this->user->password;
                $valid = password_verify($cleaned_data['password'], $hash);
                if (!$valid) {
                    $this->errors['password'] = 'Wrong password.'; 
                    return false;
                }
                $token = hash('sha512', $cleaned_data['username'].time().$_SERVER['REMOTE_ADDR']);
                $query = 'UPDATE users SET token = \''.$token.'\' WHERE username = \''.$cleaned_data['username'].'\'';
                $query = $this->db->query($query);
                if (!$query) {
                    $this->errors['database'] = 'Error on database connection.';
                    return false;
                }
                setcookie('token', $token, time() + 127800);
                $this->logged_in = true;
                return true;
            }
            return false;
        }
        
        // removes session cookie
        function logout() {
            $this->logged_in = false;
            if (isset($_COOKIE['token'])) {
                unset($_COOKIE['token']);
                setcookie('token', '', time() - 1);
            }
        }
        
        // checks if a user is logged in and updates local user
        function isLoggedIn() {
            if (!isset($_COOKIE['token']) || empty($_COOKIE['token'])) return false;
            if (!$this->logged_in) {
                $query = 'SELECT * FROM users WHERE token = \''.$_COOKIE['token'].'\';';
                $query = $this->db->query($query);
                if (!$query) {
                    $this->errors['database'] = 'Error on database connection.';
                    return false;
                }
                if ($query->num_rows < 1) {
                    $this->logout();
                    return false;
                }
                
                $this->user = $query->fetch_object();
                $token = $this->user->token;
                if ($token == $_COOKIE['token']) {
                    return true;
                }
            }
            return false;
        }
        
        // returns local user
        function user() {
            return $this->user;
        }
        
    }